#!/usr/bin/env ruby
# Id: glyphinfo.rb,v 1.1 2011/11/29 15:43:26 rjs Exp 
#
# Simple PDF glyph dumper based on PDFlib TET
#

require './TET'

# TET_char_info character types with real geometry info.
TET_CT__REAL		= 0
TET_CT_NORMAL		= 0
TET_CT_SEQ_START	= 1

# TET_char_info character types with artificial geometry info.
TET_CT__ARTIFICIAL	= 10
TET_CT_SEQ_CONT		= 10
TET_CT_INSERTED		= 12


# TET_char_info text rendering modes.
TET_TR_FILL		= 0	# fill text
TET_TR_STROKE		= 1	# stroke text (outline)
TET_TR_FILLSTROKE	= 2	# fill and stroke text
TET_TR_INVISIBLE	= 3	# invisible text
TET_TR_FILL_CLIP	= 4	# fill text and
                                # add it to the clipping path
TET_TR_STROKE_CLIP	= 5	# stroke text and
                                #  add it to the clipping path
TET_TR_FILLSTROKE_CLIP	= 6	# fill and stroke text and
                                #  add it to the clipping path
TET_TR_CLIP		= 7	# add text to the clipping path

# TET_char_info attributes
TET_ATTR_NONE      = 0x00000000
TET_ATTR_SUB       = 0x1	# subscript
TET_ATTR_SUP       = 0x2	# superscript
TET_ATTR_DROPCAP   = 0x4	# initial large letter
TET_ATTR_SHADOW    = 0x8	# shadowed text
# character before hyphenation
TET_ATTR_DEHYPHENATION_PRE       = 0x10
# hyphenation artifact, i.e. the dash
TET_ATTR_DEHYPHENATION_ARTIFACT  = 0x20
# character after hyphenation
TET_ATTR_DEHYPHENATION_POST      = 0x40


# global option list
globaloptlist = "searchpath={{../data} {../../../resource/cmap}}"

# document-specific option list
docoptlist = ""

# page-specific option list
pageoptlist = "granularity=word"


def print_color_value (outfp, tet, doc, colorid)
    # We handle only the fill color, but ignore the stroke color.
    # The stroke color can be retrieved analogously with the
    # keyword "stroke".
    
    colorinfo = tet.get_color_info(doc, colorid, "usage=fill");

    if (colorinfo.colorspaceid == -1 && colorinfo.patternid == -1)
      outfp.printf " (not filled)"
      return
    end

    outfp.printf " ("

    if (colorinfo.patternid != -1)
      patterntype =
	    tet.pcos_get_number(doc, "patterns[" + colorinfo.patternid.to_s + "]/PatternType")
      
      if (patterntype == 1)	# Tiling pattern 
          painttype =	tet.pcos_get_number(doc, "patterns[" + colorinfo.patternid.to_s + "]/PaintType")
        if (painttype == 1)
          outfp.printf "colored Pattern)"
          return  
        elsif (painttype == 2)
          outfp.printf  "uncolored Pattern, base color: "
          # FALLTHROUGH to colorspaceid output 
        end
      
      elsif (patterntype == 2)	# Shading pattern 
        shadingtype = tet.pcos_get_number(doc, "patterns[" + colorinfo.patternid.to_s + "]/Shading/ShadingType")
        outfp.printf "shading Pattern, ShadingType=%d)", shadingtype
        return;
        end
    end
    

    csname = tet.pcos_get_string(doc, "colorspaces[" + colorinfo.colorspaceid.to_s + "]/name")

    outfp.printf "%s", csname

    # Emit more details depending on the colorspace type 
    if (csname == "ICCBased")
      iccprofileid = tet.pcos_get_number(doc,"colorspaces[" + colorinfo.colorspaceid.to_s + "]/iccprofileid").to_i

      errormessage = tet.pcos_get_string(doc, "iccprofiles[" + iccprofileid.to_s + "]/errormessage")

      # Check whether the embedded profile is damaged */
      if (errormessage != "")
        outfp.printf " (%s)", errormessage
      else
		    profilename =tet.pcos_get_string(doc,"iccprofiles[" + iccprofileid.to_s + "]/profilename")
        outfp.printf " '%s'", profilename

        profilecs = tet.pcos_get_string(doc,"iccprofiles[" + iccprofileid.to_s + "]/profilecs")
        outfp.printf  " '%s'", profilecs
      end
    elsif (csname == "Separation")
      colorantname = tet.pcos_get_string(doc, "colorspaces[" + colorinfo.colorspaceid.to_s + "]/colorantname")
      outfp.printf " '%s'", colorantname
    elsif (csname == "DeviceN")
    	outfp.printf " "
      
      0.upto(colorinfo.components.length-1) do |i|
        colorantname =tet.pcos_get_string(doc,            "colorspaces[" + colorinfo.colorspaceid.to_s + "]/colorantnames[" + i.to_s + "]")

        outfp.printf "%s", colorantname

        if (i != (colorinfo.components.length-1))
          outfp.printf "/"
        end
      end
    elsif (csname == "Indexed")
      baseid =tet.pcos_get_number(doc, "colorspaces[" + colorinfo.colorspaceid.to_s + "]/baseid").to_i

      csname = tet.pcos_get_string(doc, "colorspaces[" + baseid.to_s + "]/name")

      outfp.printf " %s", csname

    end

    outfp.printf " "
    0.upto((colorinfo.components.length)-1) do |i|
    	outfp.printf"%g", colorinfo.components[i]

      if (i != colorinfo.components.length-1)
        outfp.printf "/"
      end
    end
    outfp.printf  ")"
  return 0
end

pageno = 0

begin

    if (ARGV.length != 2)
	raise("usage: glyphinfo.pl <infilename> <outfilename>\n")
    end


    tet = TET.new

    outfp = File.new(ARGV[1], "w")

    tet.set_option(globaloptlist)

    doc = tet.open_document(ARGV[0], docoptlist)

    if (doc == -1)
	raise "Error " + tet.get_errnum().to_s + " in " + tet.get_apiname() \
	    + "(): " + tet.get_errmsg()
    end

    # get number of pages in the document
    n_pages = tet.pcos_get_number(doc, "length:pages")

    # Write UTF-8 BOM
    outfp.printf "%c%c%c", 0xef, 0xbb, 0xbf

    # loop over pages in the document
    1.upto(n_pages) do |pageno|
      previouscolorid = -1
      page = tet.open_page(doc, pageno, pageoptlist)

      if (page == -1)
          raise "Error " + tet.get_errnum().to_s + " in " + tet.get_apiname()\
        + "(): " + tet.get_errmsg()
          next                        # try next page
      end

      # Administrative information
      outfp.printf "[ Document: '%s' ]\n",
          tet.pcos_get_string(doc, "filename")

      outfp.printf "[ Document options: '%s' ]\n", docoptlist

      outfp.printf "[ Page options: '%s' ]\n", pageoptlist

      outfp.printf "[ ----- Page %d ----- ]\n", pageno


      # Retrieve all text fragments
      while (text = tet.get_text(page))
          outfp.printf "[%s]\n", text  # print the retrieved text

          # Loop over all glyphs and print their details
          while (ci = tet.get_char_info(page))
        # Fetch the font name with pCOS (based on its ID)
        fontname = tet.pcos_get_string(doc,
          "fonts[" + ci.fontid.to_s + "]/name")


        # Print the character
        outfp.printf "U+%04X", ci.uv

        # ...and its ASCII representation if appropriate
        if (ci.uv >= 0x20 && ci.uv <= 0x7F)
            outfp.printf " '%c'", ci.uv
        end

        # Print font name, size, and position
        outfp.printf " %s size=%.2f x=%.2f y=%.2f",
            fontname, ci.fontsize, ci.x, ci.y
        # Print the color id 
        outfp.printf " colorid=%d", ci.colorid
        
        # Check wheather the text color changed
        if (ci.colorid != previouscolorid)
          print_color_value(outfp, tet, doc, ci.colorid)  
          previouscolorid = ci.colorid
        end
            
            
        # Examine the "type" member
        if (ci.type == TET_CT_SEQ_START)
            outfp.printf " ligature_start"
        elsif (ci.type == TET_CT_SEQ_CONT)
            outfp.printf " ligature_cont"
        # Separators are only inserted for granularity > word
        elsif (ci.type == TET_CT_INSERTED)
            outfp.printf " inserted"
        end

        # Examine the bit flags in the "attributes" member
        if (ci.attributes != TET_ATTR_NONE)
            if ((ci.attributes & TET_ATTR_SUB) == TET_ATTR_SUB)
          outfp.printf "/sub"
            end
            if ((ci.attributes & TET_ATTR_SUP) == TET_ATTR_SUP)
          outfp.printf "/sup"
            end
            if ((ci.attributes & TET_ATTR_DROPCAP) == TET_ATTR_DROPCAP)
          outfp.printf "/dropcap"
            end
            if ((ci.attributes & TET_ATTR_SHADOW) == TET_ATTR_SHADOW)
          outfp.printf "/shadow"
            end
            if ((ci.attributes & TET_ATTR_DEHYPHENATION_PRE) == TET_ATTR_DEHYPHENATION_PRE)
          outfp.printf "/dehyphenation_pre"
            end
            if ((ci.attributes & TET_ATTR_DEHYPHENATION_ARTIFACT) == TET_ATTR_DEHYPHENATION_ARTIFACT)
          outfp.printf "/dehyphenation_artifact"
            end
            if ((ci.attributes & TET_ATTR_DEHYPHENATION_POST) == TET_ATTR_DEHYPHENATION_POST)
          outfp.printf "/dehyphenation_post"
            end
        end

      outfp.printf "\n"
        end

        outfp.printf "\n"
    end

    if (tet.get_errnum() != 0)
        print("Error "+ tet.get_errnum().to_s + " in " +  \
          tet.get_apiname() + "(): on page pageno"  \
          + tet.get_errmsg() + "\n")
    end

    tet.close_page(page)
  end

  tet.close_document(doc)

rescue TETException => pe
    print pe.backtrace.join("\n") + "\n"
    print "Error [" + pe.get_errnum.to_s + "] " + pe.get_apiname + \
	                        ": " + pe.get_errmsg
    print " on page pageno" if (pageno != 0)
    print "\n"
rescue Exception => e
    print e.backtrace.join("\n") + "\n" + e.to_s + "\n"
ensure
    tet.delete() if tet
end
